﻿#RSS Reader for CS:S
import es, gamethread, playerlib, cfglib
import os, sys, random, cPickle, datetime

try:
	import feedparser
except: pass

#attributes
info = es.AddonInfo()
info.name = "RSSReader"
info.version = "0.2"
info.basename = "rssreader"
info.author = "NUDA"
info.description = "Sending the title of RSS feeds to the died players until the round end."

config = cfglib.AddonCFG(es.getAddonPath(info.basename) + "/cfg/config.cfg")

config.text("-"*10 + "How to use" + "-"*10)
config.text("1, Download Universal Feed Parser http://feedparser.org/")
config.text("2, Extract feedparser.py to")
config.text("   " + es.getAddonPath(info.basename).replace(info.basename, "_engines/python/Lib/site-packages"))
config.text("3, Add RSS feed URLs to")
config.text("   " + es.getAddonPath(info.basename) + "/cfg/feeds.cfg (e.g. 'feed=http://foo.com/bar.xml')")
config.text("4, Load this addon with es_load " + info.basename)
config.text("")
config.cvar("rssreader_random", True, "Randomize choosing the RSS feed objects(True/False)")
config.cvar("rssreader_refresh_span", 60, "The RSS feed objects refreshing span(minutes) Recommand:>60min")
config.cvar("rssreader_msg_interval", 15, "An interval messages showing up(seconds)")
config.cvar("rssreader_msg_filter", "#dead", "Who can see the messages by filter. See http://python.eventscripts.com/pages/Playerlib.getPlayerList")

config.write()

#Error
class DetectedParsingError(Exception): pass

class LoopCounter(object):
	def __init__(self, maxnum = 0):
		self.__count = range(0, maxnum) if maxnum > 0 else [0]
	def getindex(self):
		while(True):
			for x in self.__count:
				yield x

class FeedManager(object):
	def __init__(self, addrslist):
		#attributes
		self.rnd = bool(int(es.ServerVar("rssreader_random")))
		self.__refspan = int(es.ServerVar("rssreader_refresh_span"))
		#Address list
		self.__addrs = addrslist
		#Counter
		self.__feedcnt = LoopCounter(len(addrslist)).getindex()

	def getfeed(self):
		k = random.randint(0, len(self.__addrs) - 1) if self.rnd else self.__feedcnt.next()
		file = es.getAddonPath(info.basename) + "/feeds/" + self.__addrs[k].replace("http://", "").replace("/", "") + ".dump"
		fobj = 0
		
		try:
			#Get index of feed			
			if (os.path.isfile(file) and datetime.datetime.now() - datetime.datetime.fromtimestamp(os.stat(file).st_mtime) < datetime.timedelta(minutes = self.__refspan)):
				fobj = cPickle.load(open(file, "r"))
			else:
				fobj = feedparser.parse(self.__addrs[k])
				cPickle.dump(fobj, open(file, "w"))
			
			if fobj.bozo == 1:
				raise DetectedParsingError(fobj.bozo_exception)
				
		except DetectedParsingError, errhandle:
			if isinstance(errhandle.args[0], feedparser.ThingsNobodyCaresAboutButMe):
				return fobj
			
		except (cPickle.PicklingError, cPickle.UnpicklingError):
			es.dbgmsg("Error: 'Pickling/Unpickling Failed'")
			
		except TypeError:
			es.dbgmsg("Error: 'TypeError'")
		
		except:
			es.dbgmsg("Error: 'Could not be parsed'")
		
		else:
			return fobj
		
#RSS reader class
class RSSReader(object):
	#Constructer
	def __init__(self, urls):
		#define attributes
		self.__fm = FeedManager(urls)
		self.__feed = self.__fm.getfeed()
		self.__delay = es.ServerVar("rssreader_msg_interval")
		self.__filter = str(es.ServerVar("rssreader_msg_filter"))
		
		#Counter
		self.__entrycnt = LoopCounter(len(self.__feed.entries)).getindex()

	#Begin the RSS feed reading
	def begin(self):
		self.__feed = self.__fm.getfeed()
		self.__entrycnt = LoopCounter(len(self.__feed.entries)).getindex()
		self.show()
	
	#End the RSS feed reading
	def end(self):
		gamethread.cancelDelayed("readloop")
		
	#Show the RSS titles to filtered players
	def show(self):
		#Set message
		c = random.randint(0, len(self.__feed.entries) - 1) if self.__fm.rnd and len(self.__feed.entries) > 1 else self.__entrycnt.next()
		rssmsg = "[" + self.__feed.feed.title.encode(self.__feed.encoding) + "]"
		rssmsg += self.__feed.entries[c].title.encode(self.__feed.encoding)
		
		#Send a message to the players
		for playerid in playerlib.getPlayerList(self.__filter):
			es.tell(playerid, rssmsg)
		
		#Call myself
		gamethread.delayedname(self.__delay, "readloop", self.show)

#Initialize attributes when this script is loaded
def loadconfig():
	try:		
		#Load the feeds.cfg file
		rssurls = open(es.getAddonPath(info.basename) + "/cfg/feeds.cfg", "r")
		urlist = [u.replace("feed=", "").strip() for u in rssurls if u.startswith("feed=")]
		rssurls.close()
		
		if "" in urlist:
			urlist.remove("")
		
		if [""] != urlist:
			#Return Feeds URLs
			return urlist
		else:
			es.dbgmsg("RSS feed addresses not found")
		
	except:
		es.dbgmsg("Error: 'Could not open/load feeds.cfg'")
		
#Create instance of RSS Reader
reader = RSSReader(loadconfig())

#events
def load():
	config.execute()
	
def round_start(event_var):
	reader.begin()
	
def round_end(event_var):
	reader.end()